#version 430 core

layout(local_size_x = 1,
	local_size_y = 1,
	local_size_z = 1) in;

struct VertexStruct {
	vec3 position;
	vec3 normal;
	vec2 uv;
	uint numberOfNeighbors;
	uint offset;
};

layout(std430, binding = 0) buffer Vertices {
	VertexStruct vertices[];
}verts;

layout(std430, binding = 1) buffer VertexPointIndices {
	uint vertexPointIndices[];
}vpIndices;

layout(std430, binding = 2) buffer EdgePointTableOut {
	uvec4 row[];
}edgePointTableOut;

layout(binding = 3) uniform atomic_uint edgesOutCount;

uniform uint subdivisionLevel;

void main(void) {
	uint currentVertex = gl_GlobalInvocationID.x;

	for (uint i = verts.vertices[currentVertex].offset; i < verts.vertices[currentVertex].offset + verts.vertices[currentVertex].numberOfNeighbors; i++) {
		uint secondEdgeVertex = vpIndices.vertexPointIndices[i];
		if (currentVertex < secondEdgeVertex) {
			uint edgeNumber = atomicCounterIncrement(edgesOutCount);
			uint point1 = currentVertex;
			uint point2 = currentVertex;
			uint pointsFound = 0;
			for (uint j = verts.vertices[currentVertex].offset; j < (verts.vertices[currentVertex].offset + verts.vertices[currentVertex].numberOfNeighbors)/* && j != i*/; j++) {
				uint tmp = vpIndices.vertexPointIndices[j];
				for (uint u = verts.vertices[tmp].offset; u < verts.vertices[tmp].offset + verts.vertices[tmp].numberOfNeighbors; u++) {
					if (vpIndices.vertexPointIndices[u] == secondEdgeVertex) {
						if (pointsFound == 0) {
							point1 = tmp;
						}
						else {
							point2 = tmp;
						}
						pointsFound++;
					}
				}
			}
			edgePointTableOut.row[edgeNumber] = uvec4(currentVertex, secondEdgeVertex, point1, point2);
		}
	}
}